@@ -1,15 +1,14 @@
-from .util import doesMatchPattern, check_key_in_dict
+from .util import doesMatchPattern, check_key_in_dict, get_roi
 from .ReadData import REIXS
 from .edges import EdgeDict
 from .xeol import *
-from .simplemath import apply_offset, take_derivative1d
+from .simplemath import apply_offset, grid_data, apply_savgol
 import warnings
 import numpy as np
 from .parser import math_stream
-from scipy.interpolate import interp1d
 
 
-def loadSCAscans(basedir, file, x_stream, y_stream, *args, norm=True, is_XAS=False, xoffset=None, xcoffset=None, yoffset=None, ycoffset=None, background=None, deriv=None, energyloss=None, grid_x=[None, None, None]):
+def loadSCAscans(basedir, file, x_stream, y_stream, *args, norm=True, is_XAS=False, xoffset=None, xcoffset=None, yoffset=None, ycoffset=None, background=None, energyloss=None, grid_x=[None, None, None], savgol=None):
     special_streams = ['TEY', 'TFY', 'PFY', 'iPFY', 'XES', 'rXES', 'specPFY',
                        'XRF', 'rXRF', 'XEOL', 'rXEOL', 'POY', 'TOY', 'EY', 'Sample', 'Mesh']  # all special inputs
     XAS_streams = ['TEY', 'TFY', 'PFY', 'iPFY', 'specPFY', 'POY',
@@ -26,10 +25,7 @@ def loadSCAscans(basedir, file, x_stream, y_stream, *args, norm=True, is_XAS=Fal
 
             if doesMatchPattern(y_stream, ['rXES']):
                 roi = y_stream.lstrip("rXES[").rstrip("]")
-
-                roi_low = int(roi.split(":")[0])
-                roi_high = int(roi.split(":")[1])
-
+                roi_low, roi_high = get_roi(roi)
                 return data[arg].rXES(roi_low, roi_high)
 
             elif y_stream == 'XES':
@@ -37,9 +33,7 @@ def loadSCAscans(basedir, file, x_stream, y_stream, *args, norm=True, is_XAS=Fal
 
             elif doesMatchPattern(y_stream, ['rXRF']):
                 roi = y_stream.lstrip("rXRF[").rstrip("]")
-
-                roi_low = int(roi.split(":")[0])
-                roi_high = int(roi.split(":")[1])
+                roi_low, roi_high = get_roi(roi)
                 return data[arg].rXRF(roi_low, roi_high)
 
             elif y_stream == 'XRF':
@@ -58,15 +52,12 @@ def loadSCAscans(basedir, file, x_stream, y_stream, *args, norm=True, is_XAS=Fal
                     return data[arg].iPFY(iPFY_edge=roi)
 
                 else:
-                    roi_low = int(roi.split(":")[0])
-                    roi_high = int(roi.split(":")[1])
+                    roi_low, roi_high = get_roi(roi)
                     return data[arg].iPFY(iSDDLowerBound=roi_low, iSDDUpperBound=roi_high)
 
             elif doesMatchPattern(y_stream, ["specPFY"]):
                 roi = y_stream.lstrip("specPFY[").rstrip("]")
-
-                roi_low = int(roi.split(":")[0])
-                roi_high = int(roi.split(":")[1])
+                roi_low, roi_high = get_roi(roi)
                 return data[arg].specPFY(roi_low, roi_high)
 
             elif doesMatchPattern(y_stream, ["PFY"]):
@@ -76,8 +67,7 @@ def loadSCAscans(basedir, file, x_stream, y_stream, *args, norm=True, is_XAS=Fal
                     return data[arg].PFY(PFY_edge=roi)
 
                 else:
-                    roi_low = int(roi.split(":")[0])
-                    roi_high = int(roi.split(":")[1])
+                    roi_low, roi_high = get_roi(roi)
                     return data[arg].PFY(SDDLowerBound=roi_low, SDDUpperBound=roi_high)
 
             elif y_stream == 'XEOL':
@@ -85,8 +75,7 @@ def loadSCAscans(basedir, file, x_stream, y_stream, *args, norm=True, is_XAS=Fal
 
             elif doesMatchPattern(y_stream, ["rXEOL"]):
                 roi = y_stream.lstrip("rXEOL[").rstrip("]")
-                roi_low = int(roi.split(":")[0])
-                roi_high = int(roi.split(":")[1])
+                roi_low, roi_high = get_roi(roi)
 
                 return rxeol_spec(data, arg, REIXSobj, roi_low, roi_high, background_scan=background)
 
@@ -95,15 +84,14 @@ def loadSCAscans(basedir, file, x_stream, y_stream, *args, norm=True, is_XAS=Fal
 
             elif doesMatchPattern(y_stream, ["POY"]):
                 roi = y_stream.lstrip("POY[").rstrip("]")
-                roi_low = int(roi.split(":")[0])
-                roi_high = int(roi.split(":")[1])
+                roi_low, roi_high = get_roi(roi)
 
                 return poy_spec(data, arg, REIXSobj, roi_low, roi_high, background_scan=background)
 
             elif y_stream == 'EY':
                 return data[arg].sample_current
 
-            #elif y_stream == 'tey':
+            # elif y_stream == 'tey':
             #    # spec mnemonic for sample current
             #    return data[arg].sample_current
 
@@ -150,17 +138,6 @@ def loadSCAscans(basedir, file, x_stream, y_stream, *args, norm=True, is_XAS=Fal
         else:
             return get_sca_data(x_stream, data, arg)
 
-    def grid_data(x_stream, y_stream, grid):
-        xmin = grid[0]
-        xmax = grid[1]
-
-        numPoints = int(np.ceil((xmax-xmin)/grid[2])) + 1
-        new_x = np.linspace(xmin, xmax, numPoints)
-        f = interp1d(x_stream, y_stream, fill_value='extrapolate')
-        new_y = f(new_x)
-
-        return new_x, new_y
-
     data = dict()
     REIXSobj = REIXS(basedir, file)
     for arg in args:
@@ -191,17 +168,28 @@ def loadSCAscans(basedir, file, x_stream, y_stream, *args, norm=True, is_XAS=Fal
 
         data[arg].y_stream = apply_offset(
             data[arg].y_stream, yoffset, ycoffset)
+               
+        if savgol != None:
+            if isinstance(savgol,tuple):
+                if len(savgol) == 2:
+                    savgol_deriv = 0
+                elif len(savgol) == 3:
+                    savgol_deriv = savgol[2]
+                else:
+                    raise TypeError("Savgol smoothing arguments incorrect.")
+                data[arg].x_stream, data[arg].y_stream = apply_savgol(data[arg].x_stream,data[arg].y_stream,savgol[0],savgol[1],savgol_deriv)
 
-        # Take derivatives
-        if deriv != None:
-            data[arg].y_stream = take_derivative1d(
-                data[arg].y_stream, data[arg].x_stream, deriv)
-            if norm == True:
-                data[arg].y_stream = data[arg].y_stream / \
+                if norm == True:
+                    data[arg].y_stream = data[arg].y_stream / \
                     data[arg].y_stream.max()
+            else:
+                raise TypeError("Savgol smoothing arguments incorrect.")
 
         # Transforms RIXS to energy loss scale if incident energy is given
         if energyloss != None:
-            data[arg].x_stream = energyloss-data[arg].x_stream
+            if energyloss == True:
+                data[arg].x_stream = np.average(data[arg].mono_energy)-data[arg].x_stream
+            else:
+                data[arg].x_stream = energyloss-data[arg].x_stream
 
     return data
